@@ -1,13 +1,18 @@
+import boto3
 from inquirer import prompt, List, Text, Confirm, Checkbox
+from botocore.config import Config
 
 from vpc_cli.print_table import PrintTable
 from vpc_cli.create_yaml import CreateYAML
+from vpc_cli.deploy_cfn import DeployCfn
 from vpc_cli.tools import get_azs, print_figlet
-from vpc_cli.validators import name_validator, vpc_cidr_validator, subnet_count_validator, subnet_cidr_validator
+from vpc_cli.validators import name_validator, vpc_cidr_validator, subnet_count_validator, subnet_cidr_validator, \
+    stack_name_validator
 
 
 class Command:
     # variables
+    project = None
     region = None
     vpc = {
         'name': None,
@@ -18,6 +23,10 @@ class Command:
     private_subnet = []
     protected_subnet = []
     k8S_tag = False
+    flow_logs = {
+        'log-group': None,
+        'role-name': None
+    }
     igw = None
     eip = []
     nat = []
@@ -25,10 +34,12 @@ class Command:
     private_rtb = []
     protected_rtb = None
     s3_gateway_ep = None
+    dynamodb_gateway_ep = None
 
     # start command
     def __init__(self):
         print_figlet()
+        self.set_project_name()
         self.choose_region()
         self.set_vpc()
         self.set_public_subnet()
@@ -73,14 +84,18 @@ class Command:
         # skip creating s3 gateway endpoint wen all types of subnet hasn't nothing
         if len(self.public_subnet) or len(self.private_subnet) or len(self.protected_subnet):
             self.set_s3_gateway()
+            self.set_dynamodb_gateway()
         else:
-            print('Skip creating S3 Gateway Endpoint')
+            print('Skip creating S3 and DynamoDB Gateway Endpoint')
+
+        self.set_flow_logs()
 
         # print tables
         self.print_tables()
 
         # create template yaml file
         yaml_file = CreateYAML(
+            project=self.project,
             region=self.region,
             vpc=self.vpc,
             public_subnet=self.public_subnet,
@@ -92,9 +107,24 @@ class Command:
             private_rtb=self.private_rtb,
             protected_rtb=self.protected_rtb,
             nat=self.nat,
-            s3_gateway_ep=self.s3_gateway_ep
+            s3_gateway_ep=self.s3_gateway_ep,
+            dynamodb_gateway_ep=self.dynamodb_gateway_ep,
+            flow_logs=self.flow_logs
         )
         yaml_file.create_yaml()
+        DeployCfn(project=self.project, region=self.region)
+
+    def set_project_name(self):
+        questions = [
+            Text(
+                name='name',
+                message='Project name',
+                validate=lambda _, x: name_validator(x)
+            )
+        ]
+
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
+        self.project = answer['name']
 
     def choose_region(self):
         questions = [
@@ -102,28 +132,28 @@ class Command:
                 name='region',
                 message='Choose region',
                 choices=[
-                    ('us-east-1 (N. Virginia)', 'us-east-1'),
-                    ('us-east-2 (Ohio)', 'us-east-2'),
-                    ('us-west-1 (N. California)', 'us-west-1'),
-                    ('us-west-2 (Oregon)', 'us-west-2'),
-                    ('ap-south-1 (Mumbai)', 'ap-south-1'),
+                    ('us-east-1      (N. Virginia)', 'us-east-1'),
+                    ('us-east-2      (Ohio)', 'us-east-2'),
+                    ('us-west-1      (N. California)', 'us-west-1'),
+                    ('us-west-2      (Oregon)', 'us-west-2'),
+                    ('ap-south-1     (Mumbai)', 'ap-south-1'),
                     ('ap-northeast-3 (Osaka)', 'ap-northeast-3'),
                     ('ap-northeast-2 (Seoul)', 'ap-northeast-2'),
                     ('ap-southeast-1 (Singapore)', 'ap-southeast-1'),
                     ('ap-southeast-2 (Sydney)', 'ap-southeast-2'),
                     ('ap-northeast-1 (Tokyo)', 'ap-northeast-1'),
-                    ('ca-central-1 (Canada Central)', 'ca-central-1'),
-                    ('eu-central-1 (Frankfurt)', 'eu-central-1'),
-                    ('eu-west-1 (Ireland)', 'eu-west-1'),
-                    ('eu-west-2 (London)', 'eu-west-2'),
-                    ('eu-west-3 (Paris)', 'eu-west-3'),
-                    ('eu-north-1 (Stockholm)', 'eu-north-1'),
-                    ('sa-east-1 (Sao Paulo)', 'sa-east-1')
+                    ('ca-central-1   (Canada Central)', 'ca-central-1'),
+                    ('eu-central-1   (Frankfurt)', 'eu-central-1'),
+                    ('eu-west-1      (Ireland)', 'eu-west-1'),
+                    ('eu-west-2      (London)', 'eu-west-2'),
+                    ('eu-west-3      (Paris)', 'eu-west-3'),
+                    ('eu-north-1     (Stockholm)', 'eu-north-1'),
+                    ('sa-east-1      (Sao Paulo)', 'sa-east-1')
                 ]
             )
         ]
 
-        answer = prompt(questions=questions)
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
         self.region = answer.get('region')
 
     def set_vpc(self):
@@ -140,7 +170,7 @@ class Command:
             )
         ]
 
-        answer = prompt(questions=questions)
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
         self.vpc = answer
 
         # set only vpc cidr in global variable
@@ -156,7 +186,7 @@ class Command:
             )
         ]
 
-        answer = prompt(questions=questions)
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
 
         # required public subnets
         if answer['required']:
@@ -168,7 +198,7 @@ class Command:
                 )
             ]
 
-            answer = prompt(questions=questions)
+            answer = prompt(questions=questions, raise_keyboard_interrupt=True)
 
             for i in range(0, int(answer['count'])):
                 questions = [
@@ -189,7 +219,7 @@ class Command:
                     )
                 ]
 
-                subnet_answer = prompt(questions=questions)
+                subnet_answer = prompt(questions=questions, raise_keyboard_interrupt=True)
                 self.public_subnet.append(subnet_answer)
                 self.subnet_cidrs.append(subnet_answer['cidr'])
 
@@ -205,7 +235,7 @@ class Command:
             )
         ]
 
-        answer = prompt(questions=questions)
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
 
         # required private subnets
         if answer['required']:
@@ -217,7 +247,7 @@ class Command:
                 )
             ]
 
-            answer = prompt(questions=questions)
+            answer = prompt(questions=questions, raise_keyboard_interrupt=True)
 
             for i in range(0, int(answer['count'])):
                 questions = [
@@ -238,7 +268,7 @@ class Command:
                     )
                 ]
 
-                subnet_answer = prompt(questions=questions)
+                subnet_answer = prompt(questions=questions, raise_keyboard_interrupt=True)
                 self.private_subnet.append(subnet_answer)
                 self.subnet_cidrs.append(subnet_answer['cidr'])
 
@@ -254,7 +284,7 @@ class Command:
             )
         ]
 
-        answer = prompt(questions=questions)
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
 
         if answer['required']:  # required protected subnets
             questions = [
@@ -265,7 +295,7 @@ class Command:
                 )
             ]
 
-            answer = prompt(questions=questions)
+            answer = prompt(questions=questions, raise_keyboard_interrupt=True)
 
             for i in range(0, int(answer['count'])):
                 questions = [
@@ -286,7 +316,7 @@ class Command:
                     )
                 ]
 
-                subnet_answer = prompt(questions=questions)
+                subnet_answer = prompt(questions=questions, raise_keyboard_interrupt=True)
                 self.protected_subnet.append(subnet_answer)
                 self.subnet_cidrs.append(subnet_answer['cidr'])
 
@@ -302,7 +332,7 @@ class Command:
             )
         ]
 
-        answer = prompt(questions=questions)
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
         self.k8S_tag = answer['k8s-tag']
 
     def set_internet_gateway(self):
@@ -314,7 +344,7 @@ class Command:
             )
         ]
 
-        answer = prompt(questions=questions)
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
         self.igw = answer['name']
 
     def set_elastic_ip(self):
@@ -327,7 +357,7 @@ class Command:
                 )
             ]
 
-            answer = prompt(questions=questions)
+            answer = prompt(questions=questions, raise_keyboard_interrupt=True)
             self.eip.append(answer['name'])
 
     def set_nat_gateway(self):
@@ -354,7 +384,7 @@ class Command:
                 )
             ]
 
-            answer = prompt(questions=questions)
+            answer = prompt(questions=questions, raise_keyboard_interrupt=True)
             self.nat.append(answer)
 
     def set_public_rtb(self):
@@ -366,7 +396,7 @@ class Command:
             )
         ]
 
-        answer = prompt(questions=questions)
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
         self.public_rtb = answer['name']
 
     def set_private_rtb(self):
@@ -396,7 +426,7 @@ class Command:
             else:
                 pass
 
-            answer = prompt(questions=questions)
+            answer = prompt(questions=questions, raise_keyboard_interrupt=True)
             self.private_rtb.append(answer)
 
     def set_protected_rtb(self):
@@ -408,7 +438,7 @@ class Command:
             )
         ]
 
-        answer = prompt(questions=questions)
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
         self.protected_rtb = answer['name']
 
     def set_s3_gateway(self):
@@ -425,26 +455,69 @@ class Command:
             route_table_list.append({'name': self.protected_rtb})
 
         questions = [
-            Confirm(
-                name='required',
-                message='Do you want to create S3 GATEWAY ENDPOINT?',
-                default=True
+            Checkbox(
+                name='route-table',
+                message='Select S3 Gateway Endpoint Route Tables',
+                choices=[d['name'] for d in route_table_list]
             )
         ]
 
-        answer = prompt(questions=questions)
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
+        self.s3_gateway_ep = answer
 
-        if answer['required']:
-            questions = [
-                Checkbox(
-                    name='route-table',
-                    message='Select Route Tables',
-                    choices=[d['name'] for d in route_table_list]
-                )
+    def set_dynamodb_gateway(self):
+        route_table_list = []
+
+        if self.public_rtb:
+            route_table_list.append({'name': self.public_rtb})
+
+        if self.private_rtb:
+            for rtb in self.private_rtb:
+                route_table_list.append({'name': rtb['name']})
+
+        if self.protected_rtb:
+            route_table_list.append({'name': self.protected_rtb})
+        questions = [
+            Checkbox(
+                name='route-table',
+                message='Select DynamoDB Gateway Endpoint Route Tables',
+                choices=[d['name'] for d in route_table_list]
+            )
+        ]
+
+        answer = prompt(questions=questions, raise_keyboard_interrupt=True)
+        self.dynamodb_gateway_ep = answer
+
+    def set_flow_logs(self):
+        question = [
+            Confirm(
+                name='enable',
+                message='Do you want to enable VPC Flow Logs?',
+                default=True
+            )
+        ]
+        answer = prompt(questions=question, raise_keyboard_interrupt=True)
+
+        if answer.get('enable'):
+            question = [
+                Text(
+                    name='log-group-name',
+                    message='Please type the log group name',
+                    validate=lambda _, x: name_validator(x),
+                    default=f'/aws/vpc/{self.vpc["name"]}'
+                ),
+                Text(
+                    name='log-group-role-name',
+                    message='Please type the log group IAM role name',
+                    validate=lambda _, x: name_validator(x),
+                    default=f'{self.vpc["name"]}-flow-logs-role'
+                ),
             ]
-
-            answer = prompt(questions=questions)
-            self.s3_gateway_ep = answer
+            answer = prompt(questions=question, raise_keyboard_interrupt=True)
+            self.flow_logs = {
+                'log-group': answer.get('log-group-name'),
+                'role-name': answer.get('log-group-role-name'),
+            }
 
     def print_tables(self):
         print_table = PrintTable()
@@ -465,4 +538,39 @@ class Command:
         )
         print_table.print_igw(igw=self.igw)
         print_table.print_nat(nat=self.nat)
-        print_table.print_s3_ep(s3_gateway_ep=self.s3_gateway_ep)
+        print_table.print_ep(s3_gateway_ep=self.s3_gateway_ep, dynamodb_gateway_ep=self.dynamodb_gateway_ep)
+        print_table.print_flow_logs(flow_logs=self.flow_logs)
+
+    def create_stack(self):
+        questions = [
+            Confirm(
+                name='deploy',
+                message='Do you want to deploy stack in \033[1m\033[96m{}\033[0mtest?'.format(self.region),
+                default=True
+            )
+        ]
+
+        answer = prompt(questions, raise_keyboard_interrupt=True)
+
+        if answer['deploy']:
+            questions = [
+                Text(
+                    name='stack-name',
+                    message='Type your stack name',
+                    validate=lambda _, x: stack_name_validator(x, self.region)
+                )
+            ]
+
+            answer = prompt(questions, raise_keyboard_interrupt=True)
+
+            try:
+                client = boto3.client('cloudformation', config=Config(region_name=self.region))
+
+                response = client.create_stack(
+                    StackName=answer['stack-name'],
+                    TemplateBody='file://template.yaml'
+                )
+                stack_id = response['StackId']
+
+            except Exception as e:
+                print(e)
