@@ -1,4 +1,7 @@
 import re
+import boto3
+from botocore.config import Config
+from botocore.exceptions import ClientError
 from vpc_cli.tools import cidr_overlapped
 
 
@@ -31,3 +34,20 @@ def subnet_cidr_validator(text, vpc_cidr, subnet_cidrs):
                 return False
 
         return True
+
+
+def stack_name_validator(text, region):
+    if not len(text):
+        return False
+
+    else:
+        try:
+            boto3.client('cloudformation', config=Config(region_name=region)).describe_stacks(StackName=text)
+
+        except ClientError:  # stack doest
+            return True
+
+        except Exception as e:
+            print(e)
+
+            return False
